/************************************************************************
 *                                                                      *
 *    Copyright (c) 1991 Wilhelm Valder, Regine Freitag, Wolfgang Dzida *
 *                        All rights reserved                           *
 *                                                                      *
 *                                                                      *
 *    Permission to copy this program or any part of this program is    *
 *    granted in case of personal, non-commercial use only. Any use     *
 *    for profit or other commercial gain without written permissions   *
 *    of the authors is prohibited.                                     *
 *                                                                      *
 *    You may freely distribute this program to friends, colleagues,    *
 *    etc.  As a courtesy we would appreciate it if you left our names  *
 *    on the code as the authors.                                       *
 *    We also, of course, disavow any liabilities                       *
 *    for any bizarre things you may choose to do with the system.      *
 *                                                                      *
 *    We would appreciate though your letting us know about any         *
 *    extensions, modifications, project suggestions, etc.              *
 *    Please mail all bugs, comments, complaints and suggestions to:    *
 *                                                                      *
 *                                                                      *
 *    GMD (German National Research Center for Computer Science)        *
 *    Institut fuer Systemtechnik (F2G2)                                *
 *    P.O. Box 1240                                                     *
 *    Schloss Birlinghoven                                              *
 *    D-5205 St. Augustin 1, West Germany                               *
 *                                                                      *
 *    email:    dzida@kmx.gmd.dbp.de                                    *
 *              freitag@gmdzi.uunet                                     *
 *                                                                      *
 ************************************************************************/
#include <stdio.h>
#include <strings.h>
#include "listener.h"
#include <exErrors.h>
#include <signal.h>
#include <exLayout.h>
#include <exEntry.h>

static char SccsID[] = "@(#)clfuncs.c	1.2\t11/25/92";            /* sccs keyword */

extern _mh_error;

static char msg[100];
MHmenu *loadApplMenu();
MHmenu *hideApplMenu();

/* loadMacroMenu: *************************************************************
 * Let the user select one of the macros that are defined in the directory
 * indicated by the environment variable MACROPATH and load the corresponding 
 * macro into the menubar 
 *****************************************************************************/

loadMacroMenu(parms)
     RequestParm *parms;
{
  char *menu;
 
  if ((menu = selectMacroSet(parms->parent)) == NULL)
    return(-1);

  if (macroSetIsLoaded(menu)) {
    sprintf(msg, "Macro set -- %s -- already loaded; replace it ?", menu);
    if (exNotify2User(listener->listener_shell, -1, -1, msg) != ANSWERYES)
      return(-1);
    hideMacroSet(getMacroSetName(menu));
  }

  if (loadMacroSet(listener->menubar, menu) < 0) {
    switch (_mh_error) {
    case ERR_OPEN:		/* cannot open macro directory */
      sprintf(msg, "Cannot load macro set: open failure !");
      break;
    case ERR_CREATE:		/* cannot create menu descr. */
      sprintf(msg, "Cannot load macro set: cannot create menu !");
      break;
    default:
      sprintf(msg, "Cannot load macro set: unrecoverable error !");
      break;
    }
  } else			/* macro successfully loaded */
    sprintf(msg, "Macro set loaded: -- %s -- !", menu);

  eshMsg(msg);
  parms->buffer = NULL;
  return(0);
}

/* hideMacroMenu: *************************************************************
 * let the user select a displayed macro menu and hide it from the menubar;
 * do not delete the macro file.
 *****************************************************************************/

hideMacroMenu(parms)
     RequestParm *parms;
{
  register char *ptr;

  if ((ptr = selectDisplayedMacros(parms->parent)) == (char *) 0) {
    eshMsg("No macro menus currently selected !");
    return;
  }

  if (hideMacroSet(ptr) < 0)
    sprintf(msg, "Macro set -- %s -- not removed !", ptr);
  else 
    sprintf(msg, "Macro set -- %s -- removed !", ptr);

  eshMsg(msg);

  parms->buffer = NULL;
}

/* newApplication: ************************************************************
 * load a new application (not already stored in the listener data structures
 * as done by processing the init file
 *****************************************************************************/

newApplication(parms)
     RequestParm *parms;
{
  char buf[80];
  char *ptr;
  int x;
  int slot;

  ptr = exGetFilename(parms->parent, -1, -1, 
		   "Enter name and parameters: ", NULL, sizeof(buf) - 2);

  if (!ptr || (*ptr == EOS))         /* entry not created or aborted */
    return;

  strncpy(&buf[2], ptr, sizeof(buf) - 2);
  free(ptr);
  ptr = &buf[2];
  x = 1;

  switch (exNotify2User(listener->listener_shell, -1, -1, 
		      			 "Connect application via pty ?")) {
  case ANSWERYES:
    buf[x--] = '!';
    break;
  case ANSWERCANCEL:
    return;
  default:
    break;
  }
    
  buf[x] = '+';
  ptr = &buf[x];

  if ((slot = getFreeApplicationSlot(listener)) != -1) {
    processApplicationLine(listener, slot, ptr);
    adjustReadMask(listener);
    sprintf(msg, "Application -- %s -- started !", 
	                        listener->application[slot].procName);
  } else
    sprintf(msg, "Cannot start -- %s -- !", ptr);

  eshMsg(msg);
  parms->buffer = NULL;
}

/* loadApplication: ***********************************************************
 * starts a new application and provides a pty for communication with the esh 
 * main process.
 *****************************************************************************/

loadApplication(parms)
     RequestParm *parms;
{
  int indx, rc;
  char *line;

  if (! (line = getApplication(parms->parent, listener, False)))
    return(-1);
  
  indx = atoi(line);
  free(line);

  if (indx < 0 || indx > ESH_MAXAPPL)
    return(-1);

  if (! listener->application[indx].procName)
    return(-1);

  line = (char *) malloc(strlen(listener->application[indx].procLine) + 3);

  strcpy(line, "+");
  if (listener->application[indx].flags & ESH_CONNECT)
    strcat(line, "!");
  strcat(line, listener->application[indx].procLine);

  rc = processApplicationLine(listener, indx, line);
  free(line);
  adjustReadMask(listener);

  if (rc < 0) 
    sprintf(msg, "Application -- %s -- NOT started !", 
	    			listener->application[indx].procName );
  else
    sprintf(msg, "Application -- %s -- started !", 
	    			listener->application[indx].procName );

  eshMsg(msg);
  parms->buffer = NULL;
  return(0);
}
	   
/* quitApplication: ***********************************************************
 * stop an application by sending SIGHUP; the application is requested to 
 * terminate on receiving this signal but has the chance to do some 
 * cleaning up.
 *****************************************************************************/

quitApplication(parms)
     RequestParm *parms;
{
  char *str;
  int indx;

  if (! (str = getApplication(parms->parent, listener, True)))
    return;
  
  indx = atoi(str);
  if (indx < 0 || indx > ESH_MAXAPPL)
    return;

  killApplication(indx, False);

  parms->buffer = NULL;
}

/* eshQuit: *******************************************************************
 * terminate working with the esh; send a SIGHUP signal to all applications to
 * make them terminate gracefully; only the shell is send a SIGKILL signal to 
 * terminate
 *****************************************************************************/
/*ARGSUSED*/
eshQuit(parms)
     RequestParm *parms;
{
  register i;

  switch (exNotifyUser(parms->parent, -1, -1, 
		       "Do you really want to leave the esh?")) {
  case ANSWERNO:
    return;
  default:
    break;
  }

  for (i = 0; i < ESH_MAXAPPL; i++) 			/* kill all kids */
    killApplication(i, (i == SHELL) ? True : False);	
  _quitEsh();
}

/* startShell: ***************************************************************
 * start a new shell and connect it accordingly; if a shell is already running,
 * return immediately
 ****************************************************************************/

startShell(parms)
     RequestParm *parms;
{
  char *shellName, *shName, *getenv();

  if (kill(listener->application[SHELL].pid, SIGINT) == 0) {
    eshMsg("A shell subprocess is already running !");
    return;
  }
    
  if (!(shellName = getenv("SHELL"))) {
    shellName = SHELLNAME;
    shName = SHNAME;
  }
  else {
    shName = rindex(shellName, '/');
    shName++;
  }

  openShell(listener, editorLines(listener->view), 
	    editorCols(listener->view), SHELL, shellName, shName);
}
