/************************************************************************
 *                                                                      *
 *    Copyright (c) 1991 Wilhelm Valder, Regine Freitag, Wolfgang Dzida *
 *                        All rights reserved                           *
 *                                                                      *
 *                                                                      *
 *    Permission to copy this program or any part of this program is    *
 *    granted in case of personal, non-commercial use only. Any use     *
 *    for profit or other commercial gain without written permissions   *
 *    of the authors is prohibited.                                     *
 *                                                                      *
 *    You may freely distribute this program to friends, colleagues,    *
 *    etc.  As a courtesy we would appreciate it if you left our names  *
 *    on the code as the authors.                                       *
 *    We also, of course, disavow any liabilities                       *
 *    for any bizarre things you may choose to do with the system.      *
 *                                                                      *
 *    We would appreciate though your letting us know about any         *
 *    extensions, modifications, project suggestions, etc.              *
 *    Please mail all bugs, comments, complaints and suggestions to:    *
 *                                                                      *
 *                                                                      *
 *    GMD (German National Research Center for Computer Science)        *
 *    Institut fuer Systemtechnik (F2G2)                                *
 *    P.O. Box 1240                                                     *
 *    Schloss Birlinghoven                                              *
 *    D-5205 St. Augustin 1, West Germany                               *
 *                                                                      *
 *    email:    dzida@kmx.gmd.dbp.de                                    *
 *              freitag@gmdzi.uunet                                     *
 *                                                                      *
 ************************************************************************/
/***************************************************************************** 
 * this file contains the (modified or new) functions provided by the 
 * listener in order to enhance or adapt the listener editor
 * to the specific listener requirements
 *****************************************************************************/

#include <stdio.h>
#include <signal.h>
#include <strings.h>
#include <string.h>
#include "listener.h"
#include <exErrors.h>
#include <history.h>
#include <ife.h>

static char SccsID[] = "@(#)cmds.c	1.2\t11/25/92";            /* sccs keyword */

static char msg[100];
extern Listener *listener;

/* expand ********************************************************************
 * expand the current word according to the normal rules for regular file-name 
 * expressions
 *****************************************************************************/

expand(editor, cnt)
     Editor editor;
     int cnt;
{
  char *word;
  exExpandInfo *info;

  if ((word = getCurrentWord(editor)) == NULL)
    return;

  if (! (info = setExpandInfo(listener->listener_window, word, getCwd))) {
    free(word);
    return;
  }

  if (expandFileName(info, True, False)) {
    replaceCurrentWord(editor, info->retBuf);
    endlin(editor);
    putCursor(editor);
  } else {
    sprintf(msg, "File name expansion of -- %s -- failed !", word);
    eshMsg(msg);
  }

  free(word);

  if (info->retBuf)
    free(info->retBuf);
}

/* completeName: **************************************************************
 * try to complete the current word into a valid file name
 *****************************************************************************/

completeName(editor)
     Editor editor;
{
  char *word;
  exExpandInfo *info;
  char cmd[20];

  if ((word = getCurrentWord(editor)) == NULL)
    return;

  if (! (info = setExpandInfo(listener->listener_window, word, getCwd))) {
    free(word);
    return;
  }
  
  if (expandFileName(info, False, False)) {
    replaceCurrentWord(editor, info->retBuf);
    if (info->matchCnt > 1) {
      if (getCmd(editor, showCompletions, cmd))
	sprintf(msg, "No unique completion (%d)! Press %s to see the list !", 
		info->matchCnt, cmd);
      else
	sprintf(msg, "No unique completion (%d)!", info->matchCnt);
	
      eshMsg(msg);
    }
  } else {
    sprintf(msg, "File name expansion of -- %s -- failed !", word);
    eshMsg(msg);
  }

  free(word);

  if (info->retBuf)
    free(info->retBuf);
}

/* showCompletions: **********************************************************
 * present the possible completions for a partial filename in a menu 
 *****************************************************************************/

showCompletions(editor)
     Editor editor;
{
  char *word;
  exExpandInfo *info;

  if ((word = getCurrentWord(editor)) == NULL)
    return;

  if (! (info = setExpandInfo(listener->listener_window, word, getCwd))) {
    free(word);
    return;
  }
  
  if (expandFileName(info, False, True)) {
    if (info->retBuf) {
      replaceCurrentWord(editor, info->retBuf);
      endlin(editor);
      putCursor(editor);
    }
  } else {
    sprintf(msg, "File name expansion of -- %s -- failed !", word);
    eshMsg(msg);
  }

  free(word);

  if (info->retBuf)
    free(info->retBuf);
}

/* intr: *********************************************************************
 * send an interupt signal (ctrl-c) to the application 
 ****************************************************************************/

intr(editor)
     Editor editor;
{
  static char buf[] = {'\003', EOS};

  write(listener->application[SHELL].fd, buf, strlen(buf)); 
}

/* quit: *********************************************************************
 * send a quit signal (ctrl-y) to the application 
 *****************************************************************************/

quit(editor)
     Editor editor;
{
  static char buf[] = {'\031', EOS};

  write(listener->application[SHELL].fd, buf, strlen(buf)); 
}

/* stop: **********************************************************************
 * send a stop signal (ctrl-z) to the application 
 *****************************************************************************/

stop(editor)
     Editor editor;
{
  static char buf[] = {'\032', EOS};

  write(listener->application[SHELL].fd, buf, strlen(buf));
}

/* send_eof: *****************************************************************
 * send and end of file character (ctrl-d) to the shell
 ****************************************************************************/

send_eof(editor)
   Editor editor;
{
  static char buf[] = {'\004', EOS};

  write(listener->application[SHELL].fd, buf, strlen(buf));
}
/* appendHistoryEntry: ********************************************************
 * append the given command argument to the history tracer component. 
 * Communication is via the file-descriptor fd.
 *****************************************************************************/

appendHistoryEntry(fd, cmd)
     int fd;
     char *cmd;
{
  static char startCMD[] = {SOC, EOS}; 
  static char stopCMD[]  = {EOC, EOS};   /* '\n' zwischen EOC u. EOS ?*/

  if (write(fd, startCMD, strlen(startCMD)) != strlen(startCMD))
    return(-1);

  cmd[strlen(cmd) - 1] = EOS;

  if (write(fd, cmd, strlen(cmd)) != strlen(cmd))
    return(-1);

  if (write(fd, stopCMD, strlen(stopCMD)) != strlen(stopCMD))
    return(-1);

  return(0);
}

/* prevCmd: ******************************************************************
 * insert the previous command that has been stored into  the command 
 * history into the command editor buffer 
 ****************************************************************************/

prevCmd(editor, cnt)
     Editor editor;
     int cnt;
{
  return(sendHistoryTracer(hstEPC));
}

/* nextCmd: *******************************************************************
 * insert the following command that has been stored into  the command 
 *  history into the command editor buffer 
 *****************************************************************************/

nextCmd(editor, cnt)
     Editor editor;
     int cnt;
{
  return(sendHistoryTracer(hstENC));
}

/* lastCmd: *******************************************************************
 * insert the last command that has been stored into  the command 
 *  history into the command editor buffer 
 *****************************************************************************/

lastCmd(editor, cnt)
     Editor editor;
     int cnt;
{
  return(sendHistoryTracer(hstELC));
}

/* thisCmd: *******************************************************************
 * insert the current command to which the internal history pointer is set
 *****************************************************************************/

thisCmd(editor, cnt)
     Editor editor;
     int cnt;
{
  return(sendHistoryTracer(hstETC));
}

/* killApplication: ***********************************************************
 * kill the specified application by sending it a kill or hangup signal 
 * depending on the value of the paramater `forced'; then close corresponding 
 * file descriptors if open and set corresponding components in the 
 * applications structure to zero 
 *****************************************************************************/

killApplication(slot, forced)
     int slot;
     Boolean forced;
{
  if (listener->application[slot].flags & ESH_STARTED) {
    kill(listener->application[slot].pid, forced ? SIGKILL : SIGHUP);
    if (listener->application[slot].flags & ESH_CONNECT)
      close(listener->application[slot].fd);

    sendMsg(listener, listener->application[slot].pid, IFE_INFO, IFE_STOP);
    listener->application[slot].fd = -1;
    listener->application[slot].pid = -1;

    listener->application[slot].flags &= ~ESH_STARTED;

    adjustReadMask(listener);
    return(0);
  } else
    return(-1);
}
  
/* sendLine: ******************************************************************
 * send the contents of the command buffer to the application (i.e. the shell);
 * if the history tracer has been started; also send it the command(s).
 *****************************************************************************/

sendLine(editor)
     Editor editor;
{
  extern errno;
  register char *cmd;
  
  editorClearMsg(editor);		/* clear msg window */

  cmd = getBufferContents(editor);
  if (! cmd)
    return(-1);

  if (! *cmd)
    return(-1);

  if (listener->application[SHELL].fd)
    if (write(listener->application[SHELL].fd,cmd,strlen(cmd)) !=strlen(cmd)) {
      sprintf(msg, "Write failed (shell), errno = %d !", errno);
      eshMsg(msg);
      sprintf(msg, "Write failed (shell), errno = %d !", errno);
      exError("sendLine", msg, NULL);
      free(cmd);
      return(-1);
    }
  
  if (listener->application[HTRACE].fd > 0) 
    if (appendHistoryEntry(listener->application[HTRACE].fd, cmd) < 0) {
      eshMsg("Cannot append command to history tracer -- killed !");
      killApplication(HTRACE, True);
    }

  free(cmd);

  listener->cmdIndex++;
  clearBuffer(listener->command);		/* clear command buffer */
  putCursor(listener->command);
  doWritePrompt(listener);			/* write new prompt */
  
  return(0);
}

/* handleMenu: ***************************************************************
 * interprets the current token as the name of a pop-up menu and tries to 
 * locate the corresponding menu; if found the menu is called and the token 
 * is replaced by the result of the menu selection 
 ****************************************************************************/

handleMenu(editor)
     Editor editor;
{
  char *menuName, cmd[10];
  exMenu *menu;
  exMenu *getPopup();
  
  if ((menuName = getCurrentWord(editor)) == NULL) {
    eshMsg("Illegal menu name: no current token !");
    return(-1);
  }

  menu = getPopup(menuName);
  free(menuName);

  if (! menu) {
    if (getCmd(listener->command, showPopups, cmd))
  sprintf(msg, "Cannot locate popup menu --%s--! Press %s to see the list !", 
	      					menuName, cmd);
    else
      sprintf(msg, "Cannot locate popup menu -- %s -- !", menuName);

    eshMsg(msg);
    return(-1);
  }
  
  exCallMenu(menu);
  replaceCurrentWord(editor, exGetMenuReturnValue(menu));
  putCursor(editor);
  return(0);
}

/* getApplication: ***********************************************************
 * creates a menus of requested applications and lets the user select one; 
 * if started is true, a menu of already started applications is generated; 
 * otherwise a menu of not yet started applications is built.
 *****************************************************************************/

char *getApplication(parent, l, started)
     Widget parent;
     Listener *l;
     Boolean started;
{
  int cnt, i;
  static struct {char *name; int index;} applTable[ESH_MAXAPPL];
  MHmenu *mptr;
  MHitem *iptr;
  int x,y;
  char *applIndex;
  char cntStr[5];
  exMenu *applList;

  getMouse_xy(l->display, &x, &y);

  for (i = 0, cnt = 0; i < ESH_MAXAPPL; i++) {
    if (! (l->application[i].flags & ESH_OCCUPIED))
      continue;
    if (l->application[i].flags & ESH_SYS)	/* no system applications */
      continue;


    if (started) {				/* get started applications */
      if (l->application[i].flags & ESH_STARTED) {
	applTable[cnt].name  = l->application[i].procName;
	applTable[cnt].index = i;
	cnt++;
      } 
    } else {					/* get appls not yet started */
      if (! (l->application[i].flags & ESH_STARTED)) {
	applTable[cnt].name  = l->application[i].procName;
	applTable[cnt].index = i;
	cnt++;
      }
    }
  }
  
  if (cnt == 0) {
    eshMsg("No applications !");
    return(NULL);
  }

  setWorkApplication();
  
  if (! (mptr = createMenu()))
    return(NULL);
  
  insertMenu(mptr);
  
  setSelCnt(mptr, 1);
  setSelType(mptr, MDS_CNT_EQ);
  initMenu(mptr, MDS_L_POPUP, 
	         started ? "loaded applications" : "applications");
  setMenuPos(mptr, x, y, MDS_POS_ABS);
  
  mptr->m_hlp->hlpmsg = strCopy("Please select one application.");
  mptr->m_key = EOS;
  
  for (i = 0; i < cnt; i++) {				/* make the items */
    iptr = newItem(mptr, applTable[i].name, EOS);	/* create item */
    sprintf(cntStr, "%d", applTable[i].index);
    iptr->i_return->retPattern = strCopy(cntStr);
  }
  
  applList = exMakeMenu((ApplicationNode *)NULL, (exMenubar *)NULL, 
			parent, mptr);
  
  if (! applList)
    return(NULL);
  
  exCallMenu(applList);
  applIndex = strCopy(exGetMenuReturnValue(applList));
  
  deleteWorkMenu(mptr);
  freeMenu(applList);

  return(applIndex);
}

/* sendHistoryTracer: ********************************************************
 * send a string to the HistoryTracer
 *****************************************************************************/

sendHistoryTracer(m)
     char *m;
{
  if (write(listener->application[HTRACE].fd, m, strlen(m)) != strlen(m))
    return(-1);
  return(0);
}
 
/* showPopups: ***************************************************************
 * Let the user see the currently loaded popup-menus
 ****************************************************************************/

showPopups(editor, cnt)
     Editor editor;
     int cnt;
{
  char *popupList[MDS_MAXMENU];
  MHmenu *mptr;
  MHitem *iptr;
  int pcnt;
  exMenu *pList;
  int x,y;
  register i;

  setWorkApplication();
  if ((pcnt = createPopupList(popupList, MDS_MAXMENU)) < 1) {
    eshMsg("No popup-menus currently loaded !");
    return;
  }

  getMouse_xy(listener->display, &x, &y);

  if ((mptr = createMenu()) == (MHmenu *) 0) {	/* create a new menu */
    setError(ERR_CREATE);
    return;
  }
  
  initMenu(mptr, MDS_M_POPUP, "loaded popup-menus");
  insertMenu(mptr);
  setSelCnt(mptr, 1);
  setSelType(mptr, MDS_CNT_LE);
  mptr->m_hlp->hlpmsg = strCopy("Please see the loaded popup-menus !");
  
  for (i = 0; i < pcnt; i++) {
    /* create the corresponding item */
    iptr = newItem(mptr, popupList[i], EOS);
    iptr->i_return->retPattern = NULL;
  }
  
  setMenuPos(mptr, x, y, MDS_POS_ABS);
  
  if (! (pList = exMakeMenu((ApplicationNode *)NULL, (exMenubar *)NULL,
			    listener->listener_window, mptr)))
    return;

  exCallMenu(pList);
  deleteWorkMenu(mptr);
  freeMenu(pList);
}

/* insertShellOutput: *********************************************************
 * insert the specified text-line into the give editor buffer; convert 
 * multiple occurences of CR,s into one nl.
 *****************************************************************************/

insertShellOutput(line, editor)
     char *line;
     Editor editor;
{
  static char buf[512];
  Boolean cr = False;
  int i;

  if (! line || ! *line)
    return;

  for (i = 0; *line; line++) {
    switch(*line) {
    case '\r': 
      if (! cr) {
	cr = True;
	buf[i++] = '\n';
      }
      break;
    case '\n':
      if (cr)
	cr = False;
      else 
	buf[i++] = '\n';
      break;
    default:
      buf[i++] = *line;
      if (cr)
	cr = False;
      break;
    }
  }
  
  buf[i] = EOS;

  editorAppend(editor, buf);
}
