/************************************************************************
 *                                                                      *
 *    Copyright (c) 1991 Wilhelm Valder, Regine Freitag, Wolfgang Dzida *
 *                        All rights reserved                           *
 *                                                                      *
 *                                                                      *
 *    Permission to copy this program or any part of this program is    *
 *    granted in case of personal, non-commercial use only. Any use     *
 *    for profit or other commercial gain without written permissions   *
 *    of the authors is prohibited.                                     *
 *                                                                      *
 *    You may freely distribute this program to friends, colleagues,    *
 *    etc.  As a courtesy we would appreciate it if you left our names  *
 *    on the code as the authors.                                       *
 *    We also, of course, disavow any liabilities                       *
 *    for any bizarre things you may choose to do with the system.      *
 *                                                                      *
 *    We would appreciate though your letting us know about any         *
 *    extensions, modifications, project suggestions, etc.              *
 *    Please mail all bugs, comments, complaints and suggestions to:    *
 *                                                                      *
 *                                                                      *
 *    GMD (German National Research Center for Computer Science)        *
 *    Institut fuer Systemtechnik (F2G2)                                *
 *    P.O. Box 1240                                                     *
 *    Schloss Birlinghoven                                              *
 *    D-5205 St. Augustin 1, West Germany                               *
 *                                                                      *
 *    email:    dzida@kmx.gmd.dbp.de                                    *
 *              freitag@gmdzi.uunet                                     *
 *                                                                      *
 ************************************************************************/
/*****************************************************************************
 * this file contains the system defined request functions 'reqViaWnd', 
 * 'reqViaMenu' and 'reqViaForm'; all request functions (functions to be called
 * on item selection) must only have a pointer to a 'RequestParm'-structure as 
 * parameter 
 ****************************************************************************/

#include "exMenu.h"
#include "m_int.h"
#include <exErrors.h>
#include <exEdit.h>

#include <exEntry.h>

/* dsplInWnd *****************************************************************
 * display the specified message in a window; after the user has decided to, 
 * the window is cleared and deleted 
 *****************************************************************************/

dsplInWnd(parms)
     RequestParm *parms;
{
  exAlert(parms->parent, parms->x, parms->y, parms->data);
}

/* reqViaWnd *****************************************************************
 * requests the user to type a string in a window; 
 * the 'RequestParm'-structure contains:
 * - data: message to be displayed in the window
 * - x,y: coordinates where window is displayed
 * - buffer: the string typed in by the user or default value (if defined)
 * - size: size of the buffer
 * - cnt, cntType: how many tokens (separated by blanks) the string can contain
 *
 * The actual length of the buffer is returned if no errors occur, -1 otherwise
 *****************************************************************************/

reqViaWnd(parms)
     RequestParm *parms;
{
  int tcnt;
  static char msg[200];
  char *ptr;

  sprintf(msg, "%s (%s%d): ", parms->data,
      countType[(int)parms->cntType - (int)MDS_CNT_NULL - 1][1], parms->cnt);

  while (1) {			      	/* as long as illegal token count */
    ptr = exGetEntry(parms->parent, parms->x, parms->y, msg,
		                    parms->buffer, parms->size);

    if (strlen(ptr) == 0)         	/* if user aborted */
      return(-1);                     	/* indicate abort */

    tcnt = countTokens(ptr);	     	/* count the tokens */
    
    if (countCondition(parms->cntType, tcnt, parms->cnt)) {
      parms->buffer = ptr;
      return(strlen(ptr));
    }

    announceIllegalTokenCount(parms->menu, parms->parent, 
			            parms->cntType, parms->cnt, tcnt);
  }
}

/* reqViaMenu ****************************************************************
 * requests the user to select an item in a submenu; 
 *
 * the 'RequestParm'-structure contains:
 * - data: name of the menu to be called
 * - x,y: coordinates where menu is displayed
 * - buffer: the return pattern of the selected item 
 * - size: size of the buffer
 * - cnt, cntType: how many items are selectable
 *
 * The actual length of the buffer is returned if no errors occur, -1 otherwise
 ****************************************************************************/

reqViaMenu(parms)
  RequestParm *parms;
{
  MHmenu *mptr;				/* MDS menu-description */
  exMenu *m;
  int tx, ty, tc;					
  MDS_POS_TYPE tp;
  MDS_CNT_TYPE tt;
  register char *ptr;

  if ((mptr = getMenuDescrByName(parms->data)) == (MHmenu *) 0) {
    setError(ERR_NULL);
    return(-1);
  }

  tx = mptr->m_pos.x; 			/* save the stored position state */
  ty = mptr->m_pos.y; 
  tp = mptr->m_pos.posType;
  tc = mptr->i_selCnt;
  tt = mptr->i_selType;

  /* temporarily store new position information */
  setMenuPos(mptr, parms->x + MH_X_OFFSET, 
	                parms->y + MH_Y_OFFSET, MDS_POS_ABS),
  setSelCnt(mptr, parms->cnt);
  setSelType(mptr, parms->cntType);

  if (! (m = exMakeMenu(NULL, ((exMenu *) parms->menu)->menubar, 
			                  parms->parent, mptr))) {
    setError(ERR_NULL);
    return(-1);
  }

  /* restore position information to the old state */
  setMenuPos(mptr, tx, ty, tp);
  setSelCnt(mptr, tc);
  setSelType(mptr, tt);

  m->menubar = ((exMenu *) parms->menu)->menubar;

  postMenu(m);				/* display menu for user selection */

  if (! (ptr = exGetMenuReturnValue(m))) {
    freeMenu(m);
    if (m->flags & MH_ABORT)
      return(-1);
    else
      return(0);
  } else {
    free(parms->buffer);
    parms->buffer = strCopy(ptr);
    freeMenu(m);
    return(strlen(parms->buffer));
  }
}

/* reqViaForm ****************************************************************
 * not implemented yet
 *****************************************************************************/
/*ARSGUSED*/
reqViaForm(parms)
     RequestParm *parms;
{
  printf("reqViaForm: not yet imlemented\n");
}

