/************************************************************************
 *                                                                      *
 *    Copyright (c) 1991 Wilhelm Valder, Regine Freitag, Wolfgang Dzida *
 *                        All rights reserved                           *
 *                                                                      *
 *                                                                      *
 *    Permission to copy this program or any part of this program is    *
 *    granted in case of personal, non-commercial use only. Any use     *
 *    for profit or other commercial gain without written permissions   *
 *    of the authors is prohibited.                                     *
 *                                                                      *
 *    You may freely distribute this program to friends, colleagues,    *
 *    etc.  As a courtesy we would appreciate it if you left our names  *
 *    on the code as the authors.                                       *
 *    We also, of course, disavow any liabilities                       *
 *    for any bizarre things you may choose to do with the system.      *
 *                                                                      *
 *    We would appreciate though your letting us know about any         *
 *    extensions, modifications, project suggestions, etc.              *
 *    Please mail all bugs, comments, complaints and suggestions to:    *
 *                                                                      *
 *                                                                      *
 *    GMD (German National Research Center for Computer Science)        *
 *    Institut fuer Systemtechnik (F2G2)                                *
 *    P.O. Box 1240                                                     *
 *    Schloss Birlinghoven                                              *
 *    D-5205 St. Augustin 1, West Germany                               *
 *                                                                      *
 *    email:    dzida@kmx.gmd.dbp.de                                    *
 *              freitag@gmdzi.uunet                                     *
 *                                                                      *
 ************************************************************************/
/*****************************************************************************
 * This file contains application functions invoked through the menus of the 
 * menu editor; menu options can either invoke application functions only or 
 * prompt the user for input before execution of an application function. The
 * input is then parsed in the following application function. The input is
 * available in the run-time structure 'exMenu' in the array 'returnValues'.
 * The index in this array is the index of the menu option being executed. This
 * index is provided as 'data' in the 'parms'-structure in the menu definition
 * (file 'makeMenus.c').
 ****************************************************************************/

#include "editDescr.h"
#include <string.h>
#include <fcntl.h>

static char SccsID[] = "@(#)editFunct.c	1.4\t6/7/93";            /* sccs keyword */

/* ApplAccessType ************************************************************
 * application function to change application access type
 *****************************************************************************/

ApplAccessType(parms)
     RequestParm *parms;
{  
  MHmenu *mptr;
  register i;

  if (!rme->applPtr) {
    setError(ERR_NULL);
    return(-1);
  }

  if (!(mptr = getMenuDescrByName(MENUSETACCESS))) {
    setError(ERR_NULL);
    return(-1);
  }

  for (i = 0; i < mptr->itemCnt; i++)
    resetDefaultItem(mptr->i_table[i]);

  if (strcmp("readaccess", parms->data) == 0) {    /* access: read access */
    rme->applPtr->a_access &= ~MDS_CHNGACC;
    rme->applPtr->a_access |= MDS_READACC;
    setDefaultItem(mptr->i_table[0]); 
  }
  else if (strcmp("chngaccess", parms->data) == 0){/* access: change access */
    rme->applPtr->a_access &= ~MDS_READACC;
    rme->applPtr->a_access |= MDS_CHNGACC;
    setDefaultItem(mptr->i_table[1]); 
  }

  rme->applPtr->_changed = TRUE;
  fillFormWithAppl(rme->menuEditDescr, rme->applPtr);

  setWorkApplication();
  return(0);
}

/* ApplChange ***************************************************************
 * change application file, application name
 *****************************************************************************/

ApplChange(parms)
     RequestParm *parms;
{
  exMenu *m;
  char *retPtr1, *retPtr2, *setName, *saveName, *file;
  register cnt = 0;
  Arg args[5];

  m = getMenuByName(MENUSETS, EditMenuWin, WinCnt);
  retPtr1 = m->entries[atoi(parms->data)]->requests[0]->returnValue; 
 
  if (!retPtr1 || emptyStr(retPtr1))
    return(-1);     /* no setting if value is empty or consists of blanks */
                                                       /* get entry value */
  retPtr2 = Malloc(strlen(retPtr1) + 2);
  strcpy(retPtr2, retPtr1);
  strcat(retPtr2, "@");

  retPtr2 = strtok(retPtr2, "@");
  while (retPtr2)
    switch (retPtr2[0]){
    case '1':
      /* set file */
      setName = retPtr2 + 2;  /*parseStr(&retPtr2, '1', '@', 80);*/

      if (! setName || emptyStr(setName)) {
	retPtr2 = strtok(NULL, "@");
	break;       /* no setting if value is empty or consists of blanks */
      }
      
      if ((rme->applPtr->a_access & MDS_CHNGACC) == 0) { 
	checkApplAccess(rme->applPtr);
	free(retPtr2);
	setWorkApplication();
	return(-1);
      }

      if (strcmp(setName, rme->applPtr->file) == 0) {
	retPtr2 = strtok(NULL, "@");
	break;
      }

      saveName = strCopy(rme->applPtr->file);
      free(rme->applPtr->file);
      rme->applPtr->file = strCopy(setName);
      rme->applPtr->_changed = True;
      
      file = getMenuPath(rme->applPtr);

      if (open(file, O_RDONLY) >= 0) {
	sprintf(msg, "File with name <%s>\n already exists !\nDo you really want to overwrite\nthis file at next save ?", setName);
	if (exNotifyUser(rme->menuEditDescr->meShell, -1, -1, msg)
	    == ANSWERNO) {                              /* reset old name */
	  free(rme->applPtr->file);
	  rme->applPtr->file = saveName;
	  rme->applPtr->_changed = False;
	  sprintf(msg, "Original file name is retained !");
	  exAlert(rme->menuEditDescr->meShell, -1, -1, msg);
	}
	else
	  free(saveName);
      }
      free(file);

      retPtr2 = strtok(NULL, "@");
      break;
    case '2':
      /* set name */
      setName = retPtr2 + 2;  /*parseStr(&retPtr2, '2', '@', E_ENTRYNAME);*/

      if (! setName || emptyStr(setName)) {
	retPtr2 = strtok(NULL, "@");
	break;        /* no setting if value is empty or consists of blanks */
      }
	
      if ((rme->applPtr->a_access & MDS_CHNGACC) == 0) { 
	checkApplAccess(rme->applPtr);
	free(retPtr2);
	setWorkApplication();
	return(-1);
      }

      if (strcmp(setName, rme->applPtr->applicationName) == 0) {
	retPtr2 = strtok(NULL, "@");
	break;
      }

      if (getApplByName(setName)) {
	sprintf(msg, "Application with name <%s>\n already exists. Select another name!", setName);

	exAlert(rme->menuEditDescr->meShell, -1, -1, msg); 
	retPtr2 = strtok(NULL, "@");
	break;
      }

      free(rme->applPtr->applicationName);
      rme->applPtr->applicationName = strCopy(setName);
      rme->applPtr->_changed = True;

      cnt = 0;                                /* application name */
      XtSetArg(args[cnt], XmNlabelString, XmStringCreateLtoR
	   (rme->applPtr->applicationName, XmSTRING_DEFAULT_CHARSET)); cnt++;
      XtSetValues(rme->menuEditDescr->head->setName[1], args, cnt);

      newApplList((RequestParm *)0);

      retPtr2 = strtok(NULL, "@");
      break;
    default:
      retPtr2 = NULL;
      break;
    }

  free(retPtr2);
  setWorkApplication();

  return(0);
}

/* adminNewMenu **************************************************************
 * application function to internally create new menu without inserting it 
 * into the MDS
 *****************************************************************************/

adminNewMenu(parms)
     RequestParm *parms;
{
  checkSubComp(rme->menuEditDescr);

  if (! acceptMenu(TRUE)) {
    setWorkApplication();
    return(-1);
  }

  rme->findPtr = (MHmenu *) 0;
  rme->menuPtr = initialMenu(rme->menuPtr);

  fillFormWithMenu(rme->menuEditDescr, rme->menuPtr);
  fillEditPulldown(rme->menuPtr);

  setWorkApplication();
  return(0);
}

/* adminNewSet **************************************************************
 * application function to create new menu set with specified name
 *****************************************************************************/

adminNewSet(parms)
     RequestParm *parms;
{
  ApplicationNode *aptr;
  exMenu *m;
  char *ptr, *setName;

  checkSubComp(rme->menuEditDescr);

  if (! acceptMenu(TRUE)) {
    setWorkApplication();
    return(-1);
  }

  dumpChanges(rme->applPtr);

  m = getMenuByName(MENUSETS, EditMenuWin, WinCnt);
  setName = strCopy(m->entries[atoi(parms->data)]->requests[0]->returnValue);  
                                                      /* get entry value */

  if (emptyStr(setName) || strcmp(setName, UNTITLEDAPPL) == 0)
    {                                /* consists of blanks, CR, tabs etc. */
      exAlert(rme->menuEditDescr->meShell, -1, -1, 
	      "Invalid name for the new menu set!");
      setWorkApplication();
      return(-1);
    }

  if (!(aptr = createInitApplication(setName))) {
    setWorkApplication();
    return(-1);
  }

  rme->applPtr = aptr;                              /* change application */
  rme->findPtr = (MHmenu *) 0;
  rme->menuPtr = initialMenu(rme->menuPtr);

  generateApplList(ApplName);
  generateMenuList(aptr, MenuName);
  generateFuncList(aptr, FuncName, ApplFuncName);

  fillFormWithAppl(rme->menuEditDescr, rme->applPtr); /* fill in form */
  fillFormWithMenu(rme->menuEditDescr, rme->menuPtr);
  fillEditPopup(rme->applPtr, rme->menuPtr);      /* set menu default values */
  fillEditPulldown(rme->menuPtr);

  free(setName);

  setWorkApplication();
  return(0);
}

/* adminDuplSet **************************************************************
 * application function to duplicate a menu set and store it under a different
 * name; whether the current menu is saved under the original menu set or not,
 * it will be copied to the new menu set in the form it is currently shown in
 * the editor (the current menu is not automatically saved under the current 
 * menu set!); the editor will show the new menu set after completion of 
 * this function
 *****************************************************************************/

adminDuplSet(parms)
     RequestParm *parms;
{
  ApplicationNode *aptr;                      /* duplicated application node */
  MHmenu *localPtr,                           /* current menu in menu editor */
         *savePtr;                           /* menu in aptr with same name as 
						localPtr or new menu in aptr */
  exMenu *m;
  char *setName;                           /* name of duplicated application */
  register i;

  checkSubComp(rme->menuEditDescr);
  localPtr = initialMenu((MHmenu *) 0);
  copyMenuDescr(rme->menuPtr, localPtr); 
  if (! (saveMenu(rme->menuEditDescr, localPtr, TRUE))) {
    exAlert(rme->menuEditDescr->meShell, -1, -1, 
      "Menu definition incomplete;\ncannot continue to create a new menu set");
    clearMenu(localPtr);
    setWorkApplication();
    return(-1);
  }

  m = getMenuByName(MENUSETS, EditMenuWin, WinCnt);
  setName = strCopy(m->entries[atoi(parms->data)]->requests[0]->returnValue);  
                                                      /* get entry value */

  if (emptyStr(setName) || strcmp(setName, UNTITLEDAPPL) == 0)
    {                                  /* consists of blanks, CR, tabs etc. */
      exAlert(rme->menuEditDescr->meShell, -1, -1, 
	      "Invalid name for the new menu set!");
      clearMenu(localPtr);
      setWorkApplication();
      return(-1);
    }

  if (!(aptr = createInitApplication(setName))) {
    clearMenu(localPtr);
    setWorkApplication();
    return(-1);
  }

  setCurrentApplication(aptr);
  for (i = 0; i < rme->applPtr->menuCnt; i++) {
    savePtr = newMenu("@pointToMenu@");
    copyMenuDescr(rme->applPtr->m_table[i], savePtr);
  }                  /* copy all menu descriptions from current application */

  if (!(savePtr = getMenuDescrByName(localPtr->m_repr->name)))
    savePtr = newMenu("@pointToMenu@");            /* current menu in editor
						      is a new menus */

  copyMenuDescr(localPtr, savePtr);      /* if current menu is not a new menu, 
					 changes are written to new menu set */
  clearMenu(localPtr);

  dumpToFile(rme->menuEditDescr->meShell);    

  if (!acceptMenu(TRUE)) {
    setWorkApplication();
    return(-1);
  }                    /* check if changes to original menu should be saved */
  dumpChanges(rme->applPtr);

  rme->applPtr = aptr;                              /* change application */
  rme->menuPtr = initialMenu(rme->menuPtr);

  if (rme->findPtr = findFirstMenu(rme->applPtr))          
    copyMenuDescr(rme->findPtr, rme->menuPtr);     /* find first accessable
						      menu */
  generateApplList(ApplName);
  generateMenuList(aptr, MenuName);
  generateFuncList(aptr, FuncName, ApplFuncName);

  fillFormWithAppl(rme->menuEditDescr, rme->applPtr); /* fill in form */
  fillFormWithMenu(rme->menuEditDescr, rme->menuPtr);
  fillEditPopup(rme->applPtr, rme->menuPtr);      /* set menu default values */
  fillEditPulldown(rme->menuPtr);

  free(setName);

  setWorkApplication();
  return(0);
}

/* adminPreview **************************************************************
 * application function to provide menu in preview mode
 *****************************************************************************/

adminPreview(parms)
     RequestParm *parms;
{
  /* show menu in preview mode */

  if (!(saveMenu(rme->menuEditDescr, rme->menuPtr, TRUE))){
    exAlert(rme->menuEditDescr->meShell, -1, -1, 
	    "Menu definition incomplete; previewer cannot be started");
    return(-1);
  }

  if ((rme->menuEditDescr->com_flag & PREVIEWON) != 0)
    stopPreview((RequestParm *) NULL);

  Previewer(rme->menuEditDescr, rme->applPtr, rme->menuPtr);

  setWorkApplication();  
  return(0);
}

/* adminQuit *****************************************************************
 * application function to quit menu editor and destroy his windows
 ****************************************************************************/

adminQuit(parms)
     RequestParm *parms;
{
  checkSubComp(rme->menuEditDescr);

  if (! acceptMenu(TRUE)) {
    setWorkApplication();
    return(-1);
  }

  dumpChanges(rme->applPtr);

  XtUnmapWidget(rme->menuEditDescr->meShell);

  stopPreview((RequestParm *) NULL);

  /* inform interface editor *
  exSendMsg(fileno(stdout), getpid(), IFE_INFO, IFE_STOP);
  sleep(2);                     * reaction time for interface editor */ 

  XCloseDisplay(XtDisplay(rme->menuEditDescr->meShell));

  exit(0);
}

/* adminSave *****************************************************************
 * application function to save edited menu under current application node
 ****************************************************************************/

adminSave(parms)
     RequestParm *parms;
{
  setCurrentApplication(rme->applPtr);

  checkSubComp(rme->menuEditDescr);

  if (! acceptMenu(False)) {
    setWorkApplication();
    return(-1);
  }
    
  dumpChanges(rme->applPtr);

  setWorkApplication();
  return(0);
}

/* adminDuplMenu *************************************************************
 * application function to duplicate a menu (i.e. store it under a different
 * name); whether the current menu is saved or not, it will be copied in the 
 * form it is currently shown in the editor (the current menu is not 
 * automatically saved!); the editor will show the new menu set after 
 * completion of this function
 *****************************************************************************/

adminDuplMenu(parms)
     RequestParm *parms;
{
  MHmenu *localPtr;                           /* current menu in menu editor */
  exMenu *m;
  char *setName;                                  /* name of duplicated menu */

  checkSubComp(rme->menuEditDescr);
  localPtr = initialMenu((MHmenu *) 0);
  copyMenuDescr(rme->menuPtr, localPtr);
  if (! (saveMenu(rme->menuEditDescr, localPtr, TRUE))) {
    exAlert(rme->menuEditDescr->meShell, -1, -1, 
      "Menu definition incomplete;\ncannot continue to duplicate menu");
    clearMenu(localPtr);
    setWorkApplication();
    return(-1);
  }

  m = getMenuByName(MENUS, EditMenuWin, WinCnt);
  setName = strCopy(m->entries[atoi(parms->data)]->requests[0]->returnValue);  
                                                      /* get entry value */

  setCurrentApplication(rme->applPtr);
  if (emptyStr(setName) || getMenuDescrByName(setName))
    {                                  /* consists of blanks, CR, tabs etc. */
      exAlert(rme->menuEditDescr->meShell, -1, -1, 
	      "Invalid or already existing name\nfor the new menu!");
      clearMenu(localPtr);
      setWorkApplication();
      return(-1);
    }

  if (! acceptMenu(TRUE)){
    clearMenu(localPtr);
    setWorkApplication();
    return(-1);
  }                     /* check if changes to original menu should be saved */

  rme->findPtr = (MHmenu *) 0;
  rme->menuPtr = initialMenu(rme->menuPtr);
  copyMenuDescr(localPtr, rme->menuPtr);
  clearMenu(localPtr);
  free(rme->menuPtr->m_repr->name);
  rme->menuPtr->m_repr->name = setName;

  fillFormWithMenu(rme->menuEditDescr, rme->menuPtr);
  fillEditPulldown(rme->menuPtr);

  acceptMenu(False);
  dumpChanges(rme->applPtr);

  setWorkApplication();
  return(0);
}

/* adminWrite *****************************************************************
 * application function to save edited menu under selected application node
 * (option 'save menu in'); no change of application node in menu editor
 ****************************************************************************/

adminWrite(parms)
     RequestParm *parms;
{
  ApplicationNode *aptr;         /* application in which menu is to be saved */
  MHmenu *savePtr,               /* menu in aptr with same name as localPtr
				    or new menu in aptr */
         *localPtr;              /* current menu in menu editor */

  if (aptr = getApplByName(ApplName[atoi(parms->data)])) {  
    if ((aptr->a_access & MDS_CHNGACC) == 0) {   /* Zugriffsrechte pruefen */
      checkApplAccess(aptr); 
      setWorkApplication();
      return(-1);
    }
  }

  setCurrentApplication(aptr);
  if ((savePtr = 
       getMenuDescrByName(rme->menuEditDescr->head->MenuEntry->string[0])) 
      && (ReplaceNotifier(aptr, savePtr) == False)) {
    setWorkApplication();
    return(-1);
  }
	    
  if ((! savePtr) && (aptr->menuCnt == MDS_MAXMENU)) {
    sprintf(msg, "Application %s already contains\nmaximum number of menus!",
	    aptr->applicationName);
    exAlert(rme->menuEditDescr->meShell, -1, -1, msg); 
    setWorkApplication();
    return(-1);
  }
	    
  checkSubComp(rme->menuEditDescr);
  localPtr = initialMenu((MHmenu *) 0);
  copyMenuDescr(rme->menuPtr, localPtr);
  if (! (saveMenu(rme->menuEditDescr, localPtr, TRUE))) {
    exAlert(rme->menuEditDescr->meShell, -1, -1, 
	    "Menu definition incomplete; cannot be saved");
    clearMenu(localPtr);
    setWorkApplication();
    return(-1);
  }   

  if (!savePtr)
    savePtr = newMenu("@pointToMenu@");

  copyMenuDescr(localPtr, savePtr);
  clearMenu(localPtr);

  /* if menu is copied to different application node, user functions 
     are deleted */
  if ((strcmp(rme->applPtr->applicationName, aptr->applicationName) != 0) &&
      checkFunc(rme->applPtr) && delMenuUserFunct(savePtr))
    UserFunctNotifier(aptr, savePtr);

  dumpToFile(rme->menuEditDescr->meShell);    

  setWorkApplication();
  return(0);
}

/* adminUserFunct ***********************************************************
 * if option 'define function' was called user supplied application function 
 * is defined by providing a name
 *****************************************************************************/

adminUserFunct(parms)
     RequestParm *parms;
{
  register j;
  exMenu *m;
  char *setName;

  m = getMenuByName(MENUSETS, EditMenuWin, WinCnt);
  setName = strCopy(m->entries[atoi(parms->data)]->requests[0]->returnValue);
                                                         /* get entry value */

  setName = strtok(setName, "@");

  if (!setName || emptyStr(setName))
    return(-1);     /* no setting if value is empty or consists of blanks */

  if (getFuncByName(rme->applPtr, setName)) { /* function already exists */
    sprintf(msg, "This function name already exists\n for application node %s;\n select another one!", rme->applPtr->applicationName);

    exAlert(rme->menuEditDescr->meShell, -1, -1, msg); 
    return(-1);
  }
     
  j = 1;
  if (checkFunc(rme->applPtr)) 
    while (rme->applPtr->ApplFunc[j].func)
      j++;
	  
  rme->applPtr->ApplFunc[j].funcName = strCopy(setName);
  rme->applPtr->ApplFunc[j].func = (int (*)()) (j);

  rme->applPtr->_changed = TRUE;
  dumpChanges(rme->applPtr);
     
  generateFuncList(rme->applPtr, FuncName, ApplFuncName);

  free(setName);

  setWorkApplication();
  return(0);
}

/* delUserFunct **************************************************************
 * delete user supplied application function 
 *****************************************************************************/

delUserFunct(parms)
     RequestParm *parms;
{
  int count, i;

  count = getFuncByName(rme->applPtr, ApplFuncName[atoi(parms->data)]);

  /* attention: check the request definitions which contain the deleted 
     function or one of the following functions */
	  
  free(rme->applPtr->ApplFunc[count].funcName);
	
  for (i = count + 1; i < MDS_MAXFUNC; i++) {
    if (rme->applPtr->ApplFunc[i].func && rme->applPtr->ApplFunc[i].funcName) {
      rme->applPtr->ApplFunc[i - 1].funcName = 
	strCopy(rme->applPtr->ApplFunc[i].funcName);
      free(rme->applPtr->ApplFunc[i].funcName);
      rme->applPtr->ApplFunc[i - 1].func = 
	rme->applPtr->ApplFunc[i].func;
    }
    else
      break;
  }		      /* fill the empty place in the functions array to 
			 avoid problems with indexing in array MH_FUNCTIONS */
  count = i - 1;      /* last function in array */
  rme->applPtr->ApplFunc[count].funcName = (char *) 0;
  rme->applPtr->ApplFunc[count].func = (int (*)()) 0;
  
  rme->applPtr->_changed = TRUE;
	     
  setWorkApplication();
  return(0);
}

/* editAccessType ************************************************************
 * application function to change menu access type
 *****************************************************************************/

editAccessType(parms)
     RequestParm *parms;
{
  register i;
  Boolean viewMode;
  meditWindow *editw = rme->menuEditDescr;

  if (!rme->menuPtr) {
    setError(ERR_NULL);
    return(-1);
  }

  if (strcmp("readaccess", parms->data) == 0) {          /* read access */
      rme->menuPtr->m_flags = MDS_READACC;
      viewMode = TRUE;
    }
    else if (strcmp("chngaccess", parms->data) == 0) {   /* change access */
      rme->menuPtr->m_flags = MDS_CHNGACC;
      viewMode = False;
    }

  switchViewMode(editw->head->MenuEntry->entry->entryEditor, viewMode);    
  for (i = 0; i < editw->form->entryCnt; i++) {
    if (viewMode) {
      setViewMode(editw->form->ItemEntries[i]->entry->entryEditor);
      setViewMode(editw->form->ReturnEntries[i]->entry->entryEditor);
    } else {
      resetViewMode(editw->form->ItemEntries[i]->entry->entryEditor);
      resetViewMode(editw->form->ReturnEntries[i]->entry->entryEditor);
    }
  }
  setWorkApplication();
  return(0);
}

/* editMenuActMask ************************************************************
 * application function to edit menu activity mask
 *****************************************************************************/

editMenuActMask(parms)
     RequestParm *parms;
{
  int mx, my;
  int fw, fh;
  static int menuMask[1] = {0};

  if (!rme->menuPtr) {
    setError(ERR_NULL);
    return(-1);
  }

  fw = exFontWidth(rme->menuEditDescr->head->mFontPtr);
  fh = exFontHeight(rme->menuEditDescr->head->mFontPtr);

  getMouse_xy(XtDisplay(rme->menuEditDescr->meShell), &mx, &my);

  editActivityMask(rme->menuPtr, mx - 45 * fw, my - fh); 

  setWorkApplication();
  return(0);
}

/* editMenuHelp ***************************************************************
 * application function to edit help text
 *****************************************************************************/

editMenuHelp(parms)
     RequestParm *parms;
{
  int mx, my;
  int fw, fh;

  if (!rme->menuPtr) {
    setError(ERR_NULL);
    return(-1);
  }

  fw = exFontWidth(rme->menuEditDescr->head->mFontPtr);
  fh = exFontHeight(rme->menuEditDescr->head->mFontPtr);
  
  getMouse_xy(XtDisplay(rme->menuEditDescr->meShell), &mx, &my);
  createHelpForm(rme->menuEditDescr, rme->menuPtr, mx - 5 * fw, my - 6 * fh);

  subCompState(rme->menuEditDescr, 2, True);

  setWorkApplication();
  return(0);
}

/* editMenuPos1 *************************************************************
 * application function to change menu position
 *****************************************************************************/

editMenuPos1(parms)
     RequestParm *parms;
{  
  exMenu *m;
  char *setName, *xvalue, *yvalue;

  if (!rme->menuPtr) {
    setError(ERR_NULL);
    return(-1);
  }

  m = getMenuByName(MENUPOSITION, EditMenuWin, WinCnt);
  setName = strCopy(m->entries[atoi(parms->data)]->requests[0]->returnValue);
                                                         /* get entry value */
 
  xvalue = parseStr(&setName, '(', ',', 10);             /* x-position */
  yvalue = parseStr(&setName, ',', ')', 10);             /* y-position */

  if (!xvalue || emptyStr(xvalue) || 
      !yvalue || emptyStr(yvalue)) {
    exAlert(rme->menuEditDescr->meShell, -1, -1, 
	    "Invalid or incomplete position");
    return(-1);
  }

  setMenuCoordinates(rme->menuPtr, atoi(xvalue), atoi(yvalue));
  
  free(xvalue);
  free(yvalue);
  free(setName);
 
  setWorkApplication();
  return(0);
}

/* editMenuPos2 ***************************************************************
 * application function to change menu position type
 *****************************************************************************/

editMenuPos2(parms)
     RequestParm *parms;
{
  if (!rme->menuPtr) {
    setError(ERR_NULL);
    return(-1);
  }

  if (strcmp("abscoord", parms->data) == 0)      /* position type: absolute */
    setMenuPosType(rme->menuPtr, MDS_POS_ABS);
  else if (strcmp("relcoord", parms->data) == 0) /* position type: relative */
    setMenuPosType(rme->menuPtr, MDS_POS_CREL);

  setWorkApplication();
  return(0);
}

/* editMenuType **************************************************************
 * application function to change menu type
 *****************************************************************************/

editMenuType(parms)
     RequestParm *parms;
{  
  if (!rme->menuPtr) {
    setError(ERR_NULL);
    return(-1);
  }
                                                       /* menu type:*/
  if (strcmp("popupmenu", parms->data) == 0)           /* popup menu*/
    setMenuType(rme->menuPtr, MDS_M_POPUP);
  else if (strcmp("pulldownmenu", parms->data) == 0)   /* pulldown menu*/
    setMenuType(rme->menuPtr, MDS_M_PULLD);
  else if (strcmp("popuplist", parms->data) == 0)      /* popup list */
    setMenuType(rme->menuPtr, MDS_L_POPUP);
  else if (strcmp("pulldownlist", parms->data) == 0)   /* pulldown list*/
    setMenuType(rme->menuPtr, MDS_L_PULLD);
 
  setWorkApplication();
  return(0);
}

/* editRequest ***************************************************************
 * application function to edit menu requests by means of a special editor
 *****************************************************************************/

editRequest(parms)
     RequestParm *parms;
{
  int mx, my;
  int fw, fh;

  if (!rme->menuPtr) {
    setError(ERR_NULL);
    return(-1);
  }

  fw = exFontWidth(rme->menuEditDescr->head->mFontPtr);
  fh = exFontHeight(rme->menuEditDescr->head->mFontPtr);

  getMouse_xy(XtDisplay(rme->menuEditDescr->meShell), &mx, &my);
  editReqForm1(rme->menuPtr, mx, my + 10 * fh);

  setWorkApplication();
  return(0);
}

/* editSelCnt ***************************************************************
 * application function to edit menu selection count
 *****************************************************************************/

editSelCnt(parms)
     RequestParm *parms;
{
  exMenu *m;
  char *setName;

  if (!rme->menuPtr) {
    setError(ERR_NULL);
    return(-1);
  }

  m = getMenuByName(ITEMSELECTION, EditMenuWin, WinCnt);
  setName = strCopy(m->entries[atoi(parms->data)]->requests[0]->returnValue);
                                                        /* get entry value */

  if (!setName || emptyStr(setName)) {
    exAlert(rme->menuEditDescr->meShell, -1, -1, "Invalid selection count");
    return(-1);
  }              /* no setting if value is empty or consists of blanks */

  setSelCnt(rme->menuPtr, atoi(setName));
  free(setName);

  setWorkApplication();
  return(0);
}

/* editSelType ***************************************************************
 * application function to change menu selection type
 *****************************************************************************/

editSelType(parms)
     RequestParm *parms;
{
  if (!rme->menuPtr) {
    setError(ERR_NULL);
    return(-1);
  }
                                                      /* selection type: */ 
  if (strcmp("equal", parms->data) == 0)              /* equal */
    setSelType(rme->menuPtr, MDS_CNT_EQ);
  else if (strcmp("less", parms->data) == 0)          /* less */
    setSelType(rme->menuPtr, MDS_CNT_LT);
  else if (strcmp("eqless", parms->data) == 0)        /* equal or less */
    setSelType(rme->menuPtr, MDS_CNT_LE);
  else if (strcmp("greater", parms->data) == 0)       /* greater */
    setSelType(rme->menuPtr, MDS_CNT_GT);
  else if (strcmp("eqgreater", parms->data) == 0)     /* equal or greater */  
    setSelType(rme->menuPtr, MDS_CNT_GE);

  setWorkApplication();
  return(0);
}

/* listUserFunct **************************************************************
 * show list of  user supplied application function; selection has no effect 
 *****************************************************************************/

listUserFunct(parms)
     RequestParm *parms;
{
  setWorkApplication();
  return(0);
}

/* meCopyMenu **************************************************************
 * copy menu to specified application; no switch of visible application node
 * in menu editor (vgl. adminWrite)
 ***************************************************************************/

meCopyMenu(parms)
     RequestParm *parms;
{
  MHmenu *mptr,
         *cPtr,                  /* menu to copy */
         *listPtr;               /* menu with application names */
  exMenu *listmenu;
  ApplicationNode *aptr;         /* application to which menu is copied */
  char *ptr;

  mptr = getMenuDescrByName(APPLCOPYMENU);
  listPtr = buildMenuFromStrings(" copy to menu set", MDS_L_POPUP, ApplName, 
				 (char *) 0, mptr->itemCnt); 
  setMenuFlag(listPtr, MDS_CHNGACC);
  listPtr->maxItem = 10; 

  listmenu = exMakeMenu(currentApplication(), NULL, 
                        rme->menuEditDescr->meShell, listPtr);
  exCallMenu(listmenu);

  if (ptr = exGetMenuReturnValue(listmenu)) {    /* index of application */

    clearMenu(listPtr);
    if (!(aptr = getApplByName(ApplName[atoi(ptr)]))) {
      sprintf(msg, "Specified menu set not found!"); 
      exAlert(rme->menuEditDescr->meShell, -1, -1, msg);
      setWorkApplication();
      return(-1);
    }

    setCurrentApplication(rme->applPtr);
    cPtr = getMenuDescrByName(MenuName[atoi(parms->data)]);

    setCurrentApplication(aptr);
    if ((mptr = getMenuDescrByName(cPtr->m_repr->name)) 
	&& (ReplaceNotifier(aptr, mptr) == False)) {
      setWorkApplication();
      return(-1);
    }

    if ((! mptr) && (aptr->menuCnt == MDS_MAXMENU)) {
      sprintf(msg, "Application %s already contains\nmaximum number of menus!",
	    aptr->applicationName);
      exAlert(rme->menuEditDescr->meShell, -1, -1, msg);
      setWorkApplication();
      return(-1);
    }

    if (!mptr)
      mptr = newMenu("@pointToMenu@");

    copyMenuDescr(cPtr, mptr);

    /* if menu is copied to different application node, user functions 
       are deleted */
    if ((strcmp(rme->applPtr->applicationName, aptr->applicationName) != 0) &&
	checkFunc(rme->applPtr) && delMenuUserFunct(cPtr))
      UserFunctNotifier(aptr, cPtr);

    dumpToFile(rme->menuEditDescr->meShell);    
  }

  setWorkApplication();
  return(0);
}

/* meDeleteApplication *******************************************************
 * delete selected application; create new application list menu afterwards
 * by executing newApplList after termination of this menu
 *****************************************************************************/

meDeleteApplication(parms)
     RequestParm *parms;
{
  ApplicationNode *delPtr, *nPtr;
  register i = 0;
  int index;
  char *path;

  index = atoi(parms->data);

  if (!(delPtr = getApplByName(ApplName[index]))) { 
    setError(ERR_NULL);
    return(-1);
  }

  if ((delPtr->a_access & MDS_CHNGACC) == 0) {  
    sprintf(msg, "You are not allowed\nto delete application <%s>\n because you have no change access\nfor this application node!", delPtr);

    exAlert(rme->menuEditDescr->meShell, -1, -1, msg);
    return(-1);
  }

  while ((i < delPtr->menuCnt) && (! delPtr->m_table[i]))
    i++;                                    /* find first defined menu */

  if (i < delPtr->menuCnt) 
    sprintf(msg, "There are still menus defined\n under application node <%s>;\n do you really want to\n delete this application node?", 
	    delPtr->applicationName);
  else 
    sprintf(msg, "Do you really want to\ndelete application node <%s>?", 
	    delPtr->applicationName);

  if (exNotifyUser(rme->menuEditDescr->meShell, -1, -1, msg) == ANSWERNO) 
    return(-1);                       /* application node is not deleted */

  if (strcmp(rme->applPtr->applicationName, delPtr->applicationName) == 0) {
                         /* application to be deleted is current application; 
			   load previous or next user application as default */
    checkReqEdit(rme->menuEditDescr);

    /* search for next or previous user application as default */
    if (ApplName[index + 1])                  /* take next application */
      index += 1; 
    else                                      /* take previous application */
      index -= 1;

    if (index >= 0) {
      if (nPtr = getApplByName(ApplName[index])) {
	rme->applPtr = nPtr;
	if (rme->findPtr = findFirstMenu(rme->applPtr))          
	  copyMenuDescr(rme->findPtr, rme->menuPtr); 
      }     
    }
    else {                                  /* last application is deleted */
      rme->applPtr = createInitApplication(UNTITLEDAPPL);
      rme->findPtr = (MHmenu *) 0;
      rme->menuPtr = initialMenu(rme->menuPtr);
    }

    if ((! generateMenuList(rme->applPtr, MenuName)) 
	|| (! generateFuncList(rme->applPtr, FuncName, ApplFuncName)))
      return(-1);

    fillFormWithAppl(rme->menuEditDescr, rme->applPtr);    /* fill in form */
    fillFormWithMenu(rme->menuEditDescr, rme->menuPtr);  

    fillEditPopup(rme->applPtr, rme->menuPtr);    /* set menu default values */
    fillEditPulldown(rme->menuPtr);
  }

  if (path = getMenuPath(delPtr)) 
    if (unlink(path) == -1)           /* delete file */
      setError(ERR_ACCESS);
  free(path);

  removeApplNode(delPtr);                   /* delete application */

  setWorkApplication();
  return(0);
}

/* newApplList **************************************************************
 * creates new menu with application names
 ***************************************************************************/

newApplList(parms)
     RequestParm *parms;
{
  
  generateApplList(ApplName);
  adjustDefaultAppl(rme->applPtr);

  setWorkApplication();
  return(0);
}

/* newMenuList **************************************************************
 * creates new menu with menu names
 ***************************************************************************/

newMenuList(parms)
     RequestParm *parms;
{
  generateMenuList(rme->applPtr, MenuName);
  adjustDefaultMenu(rme->menuPtr);

  setWorkApplication();
  return(0);
}

/* newFuncList **************************************************************
 * creates new menu with menu names
 ***************************************************************************/

newFuncList(parms)
     RequestParm *parms;
{
  generateFuncList(rme->applPtr, FuncName, ApplFuncName);

  setWorkApplication();
  return(0);
}

/* meDeleteMenu **************************************************************
 * delete specified menu from selected application node
 ***************************************************************************/

meDeleteMenu(parms)
     RequestParm *parms;
{
  MHmenu *delPtr, *nPtr;
  int index;

  index = atoi(parms->data);

  setCurrentApplication(rme->applPtr);
  delPtr = getMenuDescrByName(MenuName[index]);

  if ((delPtr->m_flags & MDS_CHNGACC) == 0) {
    sprintf(msg, "You are not allowed\nto delete menu <%s>\nbecause you have no\nchange access for this menu!", delPtr->m_repr->name);

    exAlert(rme->menuEditDescr->meShell, -1, -1, msg);
    setWorkApplication();
    return(-1);
  }

  sprintf(msg, "Do you really want\n to delete menu <%s>\n from the current menu set <%s>?\n", delPtr->m_repr->name, rme->applPtr->applicationName);
  if (exNotifyUser(rme->menuEditDescr->meShell, -1, -1, msg) == ANSWERNO) {
    setWorkApplication();
    return(-1);
  }

  if (rme->findPtr && 
      (strcmp(delPtr->m_repr->name, rme->findPtr->m_repr->name) == 0)) { 
                           /* the menu visible in the form is to be deleted */
    checkReqEdit(rme->menuEditDescr);
    deleteMenu(rme->applPtr, delPtr);

    rme->menuPtr = initialMenu(rme->menuPtr);

    /* search for next or previous menu */
    if (MenuName[index + 1])                          /* take next menu */
      index += 1;
    else                                              /* take previous menu */
      index -= 1;
 
    if (index >= 0) {
      if (nPtr = getMenuDescrByName(MenuName[index])) {
	rme->findPtr = nPtr;
	copyMenuDescr(rme->findPtr, rme->menuPtr); 	  
      }
    }
    else {                                       /* last menu is deleted */
      rme->findPtr = (MHmenu *) 0;
      rme->menuPtr = initialMenu(rme->menuPtr);
    }
    
    fillFormWithMenu(rme->menuEditDescr, rme->menuPtr);
    fillEditPulldown(rme->menuPtr);
  }
  else
    deleteMenu(rme->applPtr, delPtr);

  rme->applPtr->_changed = TRUE;
  dumpChanges(rme->applPtr);

  setWorkApplication();
  return(0);
}

/* meSelectApplication *******************************************************
 * fill in values of selected application in menu editor
 ****************************************************************************/

meSelectApplication(parms)
     RequestParm *parms;
{
  checkSubComp(rme->menuEditDescr);
      
  if (!acceptMenu(TRUE)) {
    setWorkApplication();
    return(-1);
  }

  dumpChanges(rme->applPtr);

  if (rme->applPtr = getApplByName(ApplName[atoi(parms->data)])) {    
    rme->menuPtr = initialMenu(rme->menuPtr);

    if (rme->findPtr = findFirstMenu(rme->applPtr))          
      copyMenuDescr(rme->findPtr, rme->menuPtr);     /* find first accessable
							menu */
    if ((! generateMenuList(rme->applPtr, MenuName)) 
	|| (! generateFuncList(rme->applPtr, FuncName, ApplFuncName)))
      return(-1);

    fillFormWithAppl(rme->menuEditDescr, rme->applPtr); /* fill in form */
    fillFormWithMenu(rme->menuEditDescr, rme->menuPtr);  

    fillEditPopup(rme->applPtr, rme->menuPtr);    /* set menu default values */
    fillEditPulldown(rme->menuPtr); 
  }

  setWorkApplication();
  return(0);
}

/* meSelectMenu **************************************************************
 * fill in values of selected menu in menu editor
 ****************************************************************************/

meSelectMenu(parms)
     RequestParm *parms;
{
  checkSubComp(rme->menuEditDescr);
  
  if (! acceptMenu(TRUE)){
    setWorkApplication();
    return(-1);
  }

  dumpChanges(rme->applPtr);

  rme->findPtr = (MHmenu *) 0;
  rme->menuPtr = initialMenu(rme->menuPtr); 

  setCurrentApplication(rme->applPtr);

  if (rme->findPtr = getMenuDescrByName(MenuName[atoi(parms->data)])) 
    if (checkMenuAccess(rme->findPtr)) {
      copyMenuDescr(rme->findPtr, rme->menuPtr);
      fillFormWithMenu(rme->menuEditDescr, rme->menuPtr);
      fillEditPulldown(rme->menuPtr);
    }
  
  setWorkApplication();
  return(0);
}

selectReqFunct(parms)
     RequestParm *parms;
{
  /*printf("Request function with index %s was selected\n", parms->data);*/
}

/* stopPreview ***************************************************************
 * previewer application function to quit
 *****************************************************************************/

stopPreview(parms)
     RequestParm *parms;
{
  ApplicationNode *aptr;
  int i, cnt;

  setWorkApplication();
  aptr = currentApplication();

  if ((rme->menuEditDescr->com_flag & PREVIEWON) != 0) {
    cnt = aptr->menuCnt;
    for (i = (cnt - INTERNALS); i < cnt; i++)   
      deleteMenu(aptr, aptr->m_table[i]);
    INTERNALS = 0;

    XtDestroyWidget(rme->menuEditDescr->pre_win->pShell);
    free(rme->menuEditDescr->pre_win);

    rme->menuEditDescr->com_flag &= ~PREVIEWON;
  }

  setWorkApplication();
  return(0);
}
 
/* SystemApplFlag ************************************************************
 * application function to make application a system application
 *****************************************************************************/

SystemApplFlag(parms)
     RequestParm *parms;
{  
  ApplicationNode *aptr;
  MHmenu *mptr;

  if ((!rme->applPtr) || (! parms->data)) {
    setError(ERR_NULL);
    return(-1);
  }

  if ((rme->applPtr->a_access & MDS_CHNGACC) == 0) {  /* no change access */
    checkApplAccess(rme->applPtr);

    if (mptr = getMenuDescrByName(CHANGE)) {
      if (rme->applPtr->a_access & MDS_SYS)
	setToggle(mptr->i_table[3], TRUE);
      else 
	setToggle(mptr->i_table[3], False);
    }                /* adjust toggle item; irregular because item is 
			selected but selection does not have any effect */
    return(-1);
  }

  if (strcmp(parms->data, "0") == 0) 
    rme->applPtr->a_access &= ~MDS_SYS;  /* unset system application flag */
  else
    rme->applPtr->a_access |= MDS_SYS;   /* set system application flag */
 
  rme->applPtr->_changed = TRUE;
  fillFormWithAppl(rme->menuEditDescr, rme->applPtr);    
  
  setWorkApplication();
  return(0);
}


